/* global Mousetrap,Overlay,CodeMirror */
/**
 * Objet global définissant l'espace de nom BDF
 * 
 * @namespace BDF
 */
var BDF = {};

function $$(id) {
    if (arguments.length > 1) {
        id = Array.from(arguments).join("_");
    }
    return $(document.getElementById(id)); 
}

function ID(id) {
    if (arguments.length > 1) {
        id = Array.from(arguments).join("_");
    }
    return document.getElementById(id);
}


BDF.URL = "";
BDF.WORKING_LANG = "";
BDF.genid = 0;
BDF.genidMap = {};
BDF.jsonDataType = "json";
BDF.langConfiguration = null;
BDF.pathConfiguration = null;
BDF.timer = null;
BDF.templateInitDone = false;
BDF.templateHelpers = {
    loc: function() {
            return BDF.Loc.get.apply(this, arguments);
        },
    genid: function (nameSpace, localKey) {
            return BDF.generateId(nameSpace, localKey);
        }
};
BDF.templateTags = {
    colon: function() {
                return BDF.escape(BDF.Loc.get("_ label.global.colon"));
        },
    clean: function () {
        return this.tagCtx.render().replace(/\s\s+/gi, "");
    }
};
BDF.templateConverters = {};
BDF.hasChangeValue = false;
BDF.unloadKey = "_ warning.global.beforeunload";
BDF.registredFunctions = {};


/*******************************************************************************
* Valeurs à redéfinir pour une utilisation de LocalStorage
*******************************************************************************/

BDF.BASE_URI = false;
BDF.MAINSTORAGE_KEY = false;


/*******************************************************************************
* BDF.Loc localisation du texte (BDF.Loc.map est remplie par BDF.Loc.add)
*******************************************************************************/

BDF.Loc = {};

BDF.Loc.map = {};

BDF.Loc.add = function (locs) {
    for(let key in locs) {
        BDF.Loc.map[key] = locs[key];
    }
};

BDF.Loc.contains = function (messageKey) {
    return BDF.Loc.map.hasOwnProperty(messageKey);
};

BDF.Loc.get = function (messageKey) {
    var argLength = arguments.length;
    if (messageKey === "") {
        let text = "";
        if (argLength > 1) {
            for(let i = 1; i < argLength; i++) {
                let arg = arguments[i];
                if (!arg) {
                    text += arg;
                }
            }
        }
        return text;
    } else if (!BDF.Loc.map.hasOwnProperty(messageKey)) {
        return "?" + messageKey + "?";
    } else {
        let text = BDF.Loc.map[messageKey];
        if (argLength > 1) {
            for(let i = 1; i < argLength; i++) {
                let arg = arguments[i];
                if (!arg) {
                    arg = "";
                }
                let p = i-1;
                text = text.replace( "{" + p + "}", arg);
            }
        }
        return text;
    }
};

BDF.Loc.escape = function () {
    return BDF.escape(BDF.Loc.get.apply(this, arguments));
};



/**
 * Lancement à la suite de fonctions.
 * Les arguments doivent être des fonctions ou un tableau de fonctions comme unique argument.
 * Toutes ces fonctions doivent accepter un seul argument une fonction de rappel : 
 * 
 * @param {Array} functionArray
 * @returns {undefined}
 */
BDF.runSerialFunctions = function (functionArray) {
    var length = arguments.length;
    var functionArray;
    if (length === 0) {
        return;
    } else if ((length === 1) && (Array.isArray(arguments[0]))) {
        functionArray = arguments[0];
    } else {
        functionArray = Array.from(arguments);
    }
    length = functionArray.length;
    if (length > 0) {
        functionArray[0](_getCallback(0));
    }
    
    function _getCallback(index) {
        if (index < (length - 1)) {
            var next = index + 1;
            return function () {
                functionArray[next](_getCallback(next));
            };
        } else {
            return false;
        }
    }
};

/**
 * Retourne une fonction lançant une suite de fonctions.
 * Cette fonction prend comme seul argument une fonction de rappel.
 * Les arguments doivent être des fonctions ou un tableau de fonctions comme unique argument.
 * Toutes ces fonctions doivent accepter un seul argument une fonction de rappel : 
 * 
 * @param {Array} functionArray
 * @returns {Function}
 */
BDF.serialFunctions = function (functionArray) {
    var length = arguments.length;
    var functionArray;
    if (length > 0) {
        if ((length === 1) && (Array.isArray(arguments[0]))) {
            functionArray = Array.from(arguments[0]);
        } else {
            functionArray = Array.from(arguments);
        }
        length = functionArray.length;
    }
    if (length === 0) {
        return function (callback) {
            if (callback) {
                callback();
            }
        };
    } else {
        return function (callback) {
            functionArray.push(function () {
                if (callback) {
                    callback();
                }
            });
            length = functionArray.length;
            functionArray[0](_getCallback(0));
        };
    }

    
    function _getCallback(index) {
        if (index < (length - 1)) {
            let next = index + 1;
            return function () {
                functionArray[next](_getCallback(next));
            };
        } else {
            return false;
        }
    }
    
};


/*******************************************************************************
* Initialisation de la configuration de la langue
*******************************************************************************/

BDF.toLabelArray = function (labelMap) {
    var labelArray = new Array();
    for(var i = 0, len = BDF.langConfiguration.workingLangArray.length; i < len; i++) {
        var lang = BDF.langConfiguration.workingLangArray[i];
        var value = "";
        if (labelMap.hasOwnProperty(lang)) {
            value = labelMap[lang];
        }
        labelArray.push({
            lang: lang,
            value: value
        });
    }
    return labelArray;
};

/*******************************************************************************
* Timer
*******************************************************************************/

BDF.startTimer = function (cronFunction, milliseconds) {
    if (!milliseconds) {
        milliseconds = 1000;
    }
    BDF.clearTimer();
    BDF.timer = setInterval(cronFunction, milliseconds);
};

BDF.clearTimer = function () {
    if (BDF.timer) {
        clearInterval(BDF.timer);
        BDF.timer = null;
    }
};

BDF.hasTimer = function () {
    if (BDF.timer) {
        return true;
    } else {
        return false;
    }
};


/*******************************************************************************
* Suivi des changements
*******************************************************************************/
BDF.hasChange = function () {
    return BDF.hasChangeValue;
};

BDF.setChange = function (value) {
    BDF.hasChangeValue = value;
};

BDF.confirmUnload = function (needed) {
    var confirmKey = "_ warning.global.beforeunload";
    var finalNeeded = true;
    if (needed) {
        if ($.isFunction(needed)) {
            finalNeeded = needed();
        } else {
            finalNeeded = needed;
        }
    } else {
        finalNeeded = BDF.hasChangeValue;
    }
    if (finalNeeded) {
        var confirmation = confirm(BDF.Loc.get(confirmKey));
        if (!confirmation) {
            return false;
        }
    }
    return true;
};

BDF.initBeforeUnload = function (options) {
    var testFunction = BDF.hasChange;
    if (options) {
        if (options.unloadKey) {
            BDF.unloadKey = options.unloadKey;
        }
        if (options.testFunction) {
            testFunction = options.testFunction;
        }
    }
    $(window).on('beforeunload', function(e) {
        if (testFunction(e)) {
            var loc = BDF.Loc.get(BDF.unloadKey);
            if (loc.length === 0) {
                loc = "Discard changes ?";
            }
            return loc;
        } else {
            return;
        }
    });
};


/*******************************************************************************
* Initialisation des gabarits Jsrender
*******************************************************************************/

BDF.addTemplateOptions = function (options) {
    if (options.helpers) {
        for(var prop in options.helpers) {
            if (BDF.templateHelpers.hasOwnProperty(prop)) {
                BDF.log("Already defined in BDF.templateHelpers: " + prop);
            }
            BDF.templateHelpers[prop] = options.helpers[prop];
        }
    }
    if (options.converters) {
        for(var prop in options.converters) {
            if (BDF.templateConverters.hasOwnProperty(prop)) {
                BDF.log("Already defined in  BDF.templateConverters: " + prop);
            }
            BDF.templateConverters[prop] = options.converters[prop];
        }
    }
    if (options.tags) {
        for(var prop in options.tags) {
            if (BDF.templateTags.hasOwnProperty(prop)) {
                BDF.log("Already defined in  BDF.templateTags: " + prop);
            }
            BDF.templateTags[prop] = options.tags[prop];
        }
    }
};

BDF.initTemplates = function () {
    if (!BDF.templateInitDone) {
        BDF.templateInitDone = true;
        if ($.templates) {
            $.views.helpers(BDF.templateHelpers);
            $.views.tags(BDF.templateTags);
            $.views.converters(BDF.templateConverters);
            $("script[type='text/x-jsrender']").each(function (index, element) {
                var $element = $(element);
                var name = $element.data("name");
                $.templates(name, $element.html());
            });
        }
        Overlay.closeTooltip = BDF.Loc.escape("_ link.global.close");
    }
};

/**
 * See https://www.jsviews.com/#tmplrender
 * 
 * @param {String} name
 * @param {Object||Array} data
 * @param {Object} context
 * @param {Boolean} noIteration
 * @returns {String}
 */
BDF.render = function (name, data, context, noIteration) {
    if (!BDF.templateInitDone) {
        BDF.log("BDF.initTemplates not called");
        return "";
    }
    var template = $.templates[name];
    if (template) {
        return template.render(data, context, noIteration);
    } else {
        BDF.log("Unknown template: " + name);
        return "";
    }
};


/*******************************************************************************
* Fonctions utilitaires
*******************************************************************************/

BDF.isError = function (data) {
    if (data.commandMessage) {
        if (data.commandMessage.type === "error") {
            alert("[ERROR] " + data.commandMessage.key + " - " +  data.commandMessage.text);
            return true;
        }
    }
    return false;
};

BDF.log = function (msg) {
    if ((console) && (console.log)) {
        console.log(msg);
    }
};

BDF.removeInArray = function (array, name) {
    var index = -1;
    for(let i = 0, len = array.length; i < len; i++) {
        if (array[i].name === name) {
            index = i;
            break;
        }
    }
    if (index !== -1) {
        array.splice(index, 1);
    }
};

BDF.insertInArray = function (array, obj) {
    var objName = obj.name;
    var index = -1;
    for(let i = 0, len = array.length; i < len; i++) {
        if (array[i].name > objName) {
            index = i;
            break;
        }
    }
    if (index !== -1) {
        array.splice(index, 0, obj);
    } else {
        array.push(obj);
    }
};

BDF.updateInArray = function (array, obj) {
    var objName = obj.name;
    for(let i = 0, len = array.length; i < len; i++) {
        if (array[i].name === objName) {
            array[i] = obj;
            break;
        }
    }
};

BDF.escape = function (text) {
    var result = "";
    for(let i = 0, len = text.length; i < len; i++) {
        let carac = text.charAt(i);
        switch (carac) {
            case '&':
                result += "&amp;";
                break;
            case '"':
                result += "&quot;";
                break;
            case '<':
                result += "&lt;";
                break;
            case '>':
                result += "&gt;";
                break;
            case '\'':
                result += "&#x27;";
                break;
            default:
                result += carac;
        }
    }
    return result;
};

BDF.to$elements = function (jqArgument) {
    if (jqArgument.jquery) {
        return jqArgument;
    } else {
        return $(jqArgument);
    }
};

BDF.checkFormChange = function (jqArgument) {
    var $elements = BDF.to$elements(jqArgument);
    var change = false;
    var $inputs = $elements.find("input");
    for(let i = 0; i < $inputs.length; i++) {
        let input = $inputs[i];
        if (!input.name) {
            continue;
        }
        if ((input.type === 'checkbox') || (input.type === 'radio')) {
            change = (input.checked !== input.defaultChecked);
            
        } else if (input.type === 'file') {
            if (input.value) {
                change = true;
            }
        } else if ((input.type === 'text') || (input.type === 'password')) {
            change = (input.value !== input.defaultValue);
        }
        if (change) {
            return true;
        }
    }
    var $selects = $elements.find("select");
    for(let i = 0; i < $selects.length; i++) {
        let select = $selects[i];
        if (!select.name) {
            continue;
        }
        if (select.options.length === 0) {
            continue;
        }
        let defaultSelectedIndex = 0;
        for(let option of select.options) {
            if (option.defaultSelected) {
                defaultSelectedIndex = option.index;
                break;
            }
        }
        if (select.selectedIndex !== defaultSelectedIndex) {
            change = true;
        }
        if (change) {
            return true;
        }
    }
    var $textareas = $elements.find("textarea");
    for(let i = 0; i < $textareas.length; i++) {
        let textarea = $textareas[i];
        if (!textarea.name) {
            continue;
        }
        if (textarea.value !== textarea.defaultValue) {
            return true;
        }
    }
    return false;
};

BDF.updateDefaultValues = function (jqArgument) {
    var $elements = BDF.to$elements(jqArgument);
    $elements.find("input").each(function (index, input) {
        if (input.name) {
            if ((input.type === 'checkbox') || (input.type === 'radio')) {
                input.defaultChecked = input.checked;
            } else if (input.type === 'file') {
                input.value = null;
            } else if ((input.type === 'text') || (input.type === 'password')) {
                input.defaultValue = input.value;
            }
        }
    });
    $elements.find("select").each(function (index, select) {
        if (select.name) {
            for(let option of select.options) {
                option.defaultSelected = option.selected;
            }
        }
    });
    $elements.find("textarea").each(function (index, textarea) {
        if (textarea.name) {
            textarea.defaultValue = textarea.value;
        }
        
    });
    return false;
};

BDF.generateId = function (nameSpace, localKey) {
    var genIdKey = false;
    if ((nameSpace) && (localKey)) {
        genIdKey = nameSpace + ":" + localKey;
        if (BDF.genidMap.hasOwnProperty(genIdKey)) {
            return BDF.genidMap[genIdKey];
        }
    }
    var newId = BDF.genid + 1;
    BDF.genid = newId;
    var id = "js-";
    if (newId < 10) {
        id += "0";
    } 
    if (newId < 100) {
        id += "0";
    }
    if (newId < 1000) {
        id += "0";
    }
    id = id + newId; 
    if (genIdKey) {
        BDF.genidMap[genIdKey] = id;
    }
    return id;
};

BDF.attrMapToString = function (attrMap) {
    var text = "";
    for(var prop in attrMap) {
        text += prop;
        text += "=";
        var array = attrMap[prop];
        for(var i = 0; i < array.length; i++) {
            if (i > 0) {
                text += ";";
            }
            text += _escapeAttrValue(array[i]);
        }
        text += "\n";
    }
    return text;
    
    function _escapeAttrValue(value) {
            var result = "";
        for(var i = 0; i < value.length; i++) {
            carac = value.charAt(i);
            switch (carac) {
                case ';':
                case '\\':
                    result += '\\';
                    break;
            }
            result += carac;
        }
        return result;
    };
};

BDF.getMatchingCodeMirrorMode = function (path) {
    var extension;
    var idx = path.lastIndexOf(".");
    if (idx === -1) {
        extension = path;
    } else {
        extension = path.substring(idx + 1);
    }
    switch(extension) {
        case "css":
            return "css";
        case "xml":
             return "xml";
         case "xsl":
             return "xml";
         case "js":
            return "javascript";
        case "ini":
            return "properties";
        case "properties":
            return "properties";
        case "html":
            return "xml";
        default:
            return "";
    }
};


BDF.initKeyBind = function (saveFunction) {
    if (Mousetrap) {
        Mousetrap.bind('mod+s',function(e) {
            if (e.preventDefault) {
                e.preventDefault();
            } else {
                e.returnValue = false;
            }
            saveFunction();
        });
    }
};

BDF.getSubsetName = function ($element) {
    return $element.data("subsetName");
};

BDF.getSubsetItemId = function ($element) {
    return $element.data("subsetItemId");
};


/*******************************************************************************
* Classes codeMirror
*******************************************************************************/

BDF.parseCmSpanArray = function (key) {
    var result = new Array();
    if (key.indexOf("_master_") === 0) {
        result.push({
            text: "_master_",
            cm: "cm-qualifier"
        });
        key = key.substring("_master_".length);
    }
    var idx = key.indexOf("_");
    if (idx === -1) {
        switch(key) {
            case "liage":
            case "rattachement":
                return [{
                    text: key,
                    cm: "cm-component-subset"
                }];
            default:
                return [{
                text: key,
                cm: "cm-component-field"
            }];
        }
    }
    var prefix = key.substring(0, idx);
    result.push({
        text: prefix,
        cm: _getCmClass(prefix)
    });
    var name = key.substring(idx);
    var mode, poids = false;
    idx = name.indexOf("_", 1);
    if (idx > 0) {
        mode = name.substring(idx);
        name = name.substring(0, idx);
    }
    result.push({
        text: name,
        cm: "cm-quote"
    });
    if (mode) {
        if ((/^[0-9]+$/.test(mode))) {
            poids = mode;
            mode = false;
        } else {
            idx = mode.indexOf("_");
            if (idx > 0) {
                poids = mode.substring(idx);
                mode = mode.substring(0, idx);
            }
        }
    }
    if (mode) {
        result.push({
            text: mode,
            cm: "cm-def"
        });
    }
    if (poids) {
        result.push({
            text: mode,
            cm: "cm-number"
        });
    }
    return result;
    
    
    function _getCmClass(prefix) {
        switch(prefix) {
            case "corpus":
            case "thesaurus":
            case "addenda":
            case "album":
            case "sphere":
            case "motcle":
            case "fiche":
                return "cm-component-subset";
            default:
                return "cm-component-field";
        }
    }
};

/*******************************************************************************
* Chemins divers
*******************************************************************************/

BDF.toTmpPath = function (tmpFileName) {
    return "output/tmp/" + tmpFileName;
};

BDF.getCommandUrl = function (commandKey) {
    return "http://www.fichotheque.net/" + commandKey + ".html";
};


/*******************************************************************************
* Raccourci JQuery
*******************************************************************************/

BDF.$ = function (jqArgument, properties) {
    if (!properties) {
        properties = jqArgument;
        jqArgument = null;
    }
    var query = BDF.toCssQuery(properties);
    if (jqArgument) {
         return BDF.to$elements(jqArgument).find(query);
    } else {
         return $(query);
    }
};

BDF.$children = function (jqArgument, properties) {
    return BDF.to$elements(jqArgument).children(BDF.toCssQuery(properties));
};

BDF.$parents = function (jqArgument, properties) {
    return BDF.to$elements(jqArgument).parents(BDF.toCssQuery(properties));
};

BDF.toCssQuery = function (properties) {
    var query = "";
    var elementName = false;
    var suffix = "";
    for(let key in properties) {
        let value = properties[key];
        if (!key.startsWith("_")) {
            query += _selector(BDF.toDataAttribute(key), value);
        } else if (key === "_checked") {
            if (value) {
               suffix += ":checked";
           } else {
               suffix += ":not(:checked)";
           }
        } else if (key === "_element") {
            elementName = value;
        } else {
            query += _selector(key.substring(1), value);
        }
    }
    if (elementName) {
        query = elementName + query;
    }
    query += suffix;
    return query;
    
    
    function _selector(name, value) {
        if (value === true) {
            return "[" + name + "]";
        } else {
            value = value.toString().replace("'", "\\'");
            return "[" + name + "='" + value + "']";
        }
    }
    
};

BDF.toDataAttribute = function (camelCaseString) {
    return "data-" + camelCaseString.replace(/[A-Z]/g, function (upperLetter) {
        return "-" + upperLetter.toLowerCase();
    });
};

BDF.onClick = function ($element, properties, clickFunction) {
    $element.on("click", BDF.toCssQuery(properties), clickFunction);
};


/*******************************************************************************
* CodeMirror
*******************************************************************************/

BDF.toCodeMirror = function (textArea, config) {
    var codeMirror = CodeMirror.fromTextArea(textArea, config);
    $(textArea)
            .data("codeMirrorInstance", codeMirror)
            .parents("details").on("toggle", function () {
                codeMirror.refresh();
            });
    return codeMirror;
};

BDF.refreshCodeMirror = function (jqArgument) {
    BDF.to$elements(jqArgument).find("textarea").each(function (index, element) {
        let codeMirror = $(element).data("codeMirrorInstance");
        if (codeMirror) {
            codeMirror.refresh();
        }
    });
};

/*******************************************************************************
* Remplacement d'un champ de texte par trois champs date
*******************************************************************************/

BDF.replaceWithDateInputs = function (input) {
    var $input = BDF.to$elements(input);
    var is_number = /^[0-9]+$/;
    var value = $.trim($input.val());
    var datation = _splitDatation(value);
    if (!datation) {
        return;
    }
    var genId = BDF.generateId();
    $input.hide();
    $input.after(BDF.render("bdf:dateinputs", {
        genId: genId,
        year: datation[0],
        month: datation[1],
        day: datation[2]
    }));
    var $yearInput = $$(genId, "year");
    var $monthInput =  $$(genId, "month");
    var $dayInput =  $$(genId, "day");
    $yearInput.on('input', function () {
        var $this = $(this);
        var val = $this.val();
        var val_split = _splitDatation(val);
        if (!val_split) {
            $this.addClass("global-ErrorMessage");
        } else {
            $this.removeClass("global-ErrorMessage");
            if (val_split[1].length > 0) {
                $this.val(val_split[0]);
                $monthInput.val(val_split[1]).trigger('input');
                if (val_split[2].length > 0) {
                    $dayInput.val(val_split[2]).trigger('input');
                }
                $this.change();
            } else if (val_split[0].length === 4) {
                $monthInput.focus();
            }
        }

    }).change(_updateInput);
    $monthInput.on('input', function () {
        var $this = $(this);
        var val = $this.val();
        if (_test_month(val)) {
            $this.removeClass("global-ErrorMessage");
            if ((val.length === 2) || ((val.length === 1) && (val !== '0') && (val !== '1'))) {
                $dayInput.focus();
            }
        } else {
            $this.addClass("global-ErrorMessage");
        }
    }).keydown(function (event) {
         if ((event.key === "Backspace") && ($(this).val().length === 0)) {
            $yearInput.focus();
        }
    }).change(_updateInput);
    $dayInput.on('input', function () {
        var $this = $(this);
        if (_test_day($this.val())) {
            $this.removeClass("global-ErrorMessage");
        } else {
            $this.addClass("global-ErrorMessage");
        }
    }).keydown(function (event) {
         if ((event.key === "Backspace") && ($(this).val().length === 0)) {
            $monthInput.focus();
        }
    }).change(_updateInput);
    return genId;
    
    
    function _test_special_months(month) {
        switch(month) {
            case "T1":
            case "T2":
            case "T3":
            case "T4":
            case "S1":
            case "S2":
                return true;
            default:
                return false;
        }
    }
    
    function _test_month(month) {
        month = $.trim(month);
        var monthLength = month.length;
        if (monthLength === 0) {
            return true;
        }
        if (monthLength > 2) {
            return false;
        }
        if (_test_special_months(month)) {
            return true;
        }
        if (!is_number.test(month)) {
            return false;
        }
        if (parseInt(month) > 12) {
            return false;
        }
        return true;
    }
    
    function _test_day(day) {
        day = $.trim(day);
        var dayLength = day.length;
        if (dayLength === 0) {
            return true;
        }
        if (dayLength > 2) {
            return false;
        }
        if (!is_number.test(day)) {
            return false;
        }
        if (parseInt(day) > 31) {
            return false;
        }
        return true;
    }
    
    function _splitDatation(value) {
        value = $.trim(value);
        if (value.length === 0) {
            return ["", "", ""];
        }
        if (value === "*") {
            return ["*", "", ""];
        }
        var split = value.split("-");
        if (split.length > 3) {
            return false;
        }
        for(var i = split.length; i < 3; i++) {
            split.push("");
        }
        var year = split[0];
        var month = split[1];
        var day = split[2];
        if (year.length === 0) {
            return false;
        } else if (!is_number.test(year)) {
            return false;
        }
        if (month.length > 0) {
            if (!is_number.test(month)) {
                if (_test_special_months(month)) {
                    if (day.length > 0) {
                        return false;
                    }
                } else {
                    return false;
                }
            }
        } else if (day.length > 0) {
            if (!is_number.test(day)) {
                return false;
            }
        }
        return [year, month, day];
    }
    
    function _updateInput() {
        var newValue = "";
        var year = $.trim($yearInput.val());
        if (year.length > 0) {
            newValue = year;
            var month = $.trim($monthInput.val());
            if (month.length > 0) {
                if (month.length === 1) {
                    month = "0" + month;
                }
                newValue = newValue + "-" + month;
                var day = $.trim($dayInput.val());
                if (day.length > 0) {
                    if (day.length === 1) {
                        day = "0" + day;
                    }
                    newValue = newValue + "-" + day;
                }
            }
        }
        $input.val(newValue);
        $input.change();
    }
};


/*******************************************************************************
* BDF.registredFunctions
*******************************************************************************/

BDF.registerFunction = function (name, functionToRegister) {
    BDF.registredFunctions[name] = functionToRegister;
};

BDF.getFunction = function (name) {
    if (BDF.registredFunctions.hasOwnProperty(name)) {
        return BDF.registredFunctions[name];
    }
    return null;
};

BDF.getAppelantCallback = function (appelant) {
    return BDF.getFunction("callback:" + appelant);
};

BDF.putAppelantCallback = function (appelant, callback) {
    BDF.registerFunction("callback:" + appelant, callback);
};


/*******************************************************************************
* LocalStorage
*******************************************************************************/

BDF.getMainLocalStorage = function () {
    if (!BDF.MAINSTORAGE_KEY) {
        return false;
    }
    return BDF.getLocalStorage(BDF.MAINSTORAGE_KEY);
};

BDF.getLocalStorage = function (key) {
    if (!localStorage) {
        return false;
    }
    var chaine = localStorage.getItem(BDF.BASE_URI + "/" + key);
    if (!chaine) {
        return new Object();
    }
    return JSON.parse(chaine);
};

BDF.putMainLocalStorage = function (data) {
    if (!BDF.MAINSTORAGE_KEY) {
        return;
    }
    BDF.putLocalStorage(BDF.MAINSTORAGE_KEY, data);
};

BDF.putLocalStorage = function (key, data) {
    if (localStorage) {
        localStorage.setItem(BDF.BASE_URI + "/" + key, JSON.stringify(data));
    }
};

/*******************************************************************************
* CommandMessage
*******************************************************************************/

BDF.showCommandMessage = function (commandMessageObj) {
    if (!commandMessageObj) {
        return;
    }
    var text = "";
    if (commandMessageObj.text) {
        text = text + commandMessageObj.text;
    } else {
        text = text + commandMessageObj.key;
    }
    if (commandMessageObj.type === "error") {
        alert(text);
        return;
    }
    var html = "<div class='global-CommandMessageBox' style='top: -70px'><div>" + BDF.escape(text) + "</div></div>";
    $(html).appendTo("body").animate({
        top: -4
    }, "slow", function () {
        $(this).delay(5000).animate({
            top: -70
        }, "slow", function () {$(this).detach()})}
    );

};


/*******************************************************************************
* Overlay
*******************************************************************************/

BDF.startRunningOverlay = function () {
    return Overlay.start({
        supplementaryClasses: {
            dialog: "overlay-RunningOverlay"
        },
        header: BDF.Loc.escape("_ info.global.running"),
        content: " ",
        isWaiting: true,
        showClose: false
    });
};


/*******************************************************************************
* Traitement du JSON
*******************************************************************************/

BDF.processObject = function (data, objectName, objectCallback) {
    if (data.commandMessage) {
        if (!BDF.checkError(data)) {
            return false;
        }
    }
    if (!data.hasOwnProperty(objectName)) {
        $.error("BDF Error: " + objectName + " object is missing in json response");
        return false;
    } else {
        objectCallback(data[objectName], data);
        return true;
    }
};

BDF.checkError = function (data) {
    if (data.commandMessage) {
        if (data.commandMessage.type === "error") {
                var alertText = "";
                if (data.commandMessage.text) {
                    alertText = alertText + data.commandMessage.text;
                } else {
                    alertText = alertText + data.commandMessage.key;
                }
                if (data.commandMessage.multiErrorMessageArray) {
                    var array = data.commandMessage.multiErrorMessageArray;
                    for(var i = 0; i < array.length; i++) {
                        var obj = array[i];
                        alertText = alertText + "\n\n      - ";
                        if (obj.text) {
                            alertText = alertText + obj.text;
                        } else {
                            alertText = alertText + obj.key;
                        }
                    }
                }
                alert(alertText);
                return false;
        }
    }
    return true;
};